(function() {
  var Packet, Parser, Serializer, Structure, bufferize, hex, ieee754, noop, parsePattern, transforms;
  var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  }, __slice = Array.prototype.slice;
  parsePattern = require('./pattern').parse;
  ieee754 = require('./ieee754');
  hex = function(bytes) {
    var h;
    h = bytes.map(function(b) {
      if (b < 10) {
        return "0" + b.toString(16);
      } else {
        return b.toString(16);
      }
    });
    return h.join("");
  };
  noop = function(value) {
    return value;
  };
  Packet = (function() {
    function Packet() {
      this.packets = {};
      this.reset();
      this.fields = [];
      this.transforms = Object.create(transforms);
    }
    Packet.prototype.clone = function() {
      var copy;
      copy = new this.constructor();
      copy.packets = Object.create(packets);
      return copy;
    };
    Packet.prototype.packet = function(name, pattern, callback) {
      callback || (callback = noop);
      pattern = parsePattern(pattern);
      return this.packets[name] = {
        pattern: pattern,
        callback: callback
      };
    };
    Packet.prototype.reset = function() {
      this.bytesRead = 0;
      this.bytesWritten = 0;
      this.pattern = null;
      this.callback = null;
      return this.fields = [];
    };
    Packet.prototype.pipeline = function(pattern, value) {
      var constant, parameters, transform, _i, _j, _len, _len2, _ref, _ref2;
      if (pattern.transforms) {
        _ref = pattern.transforms;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          transform = _ref[_i];
          parameters = [];
          _ref2 = transform.parameters;
          for (_j = 0, _len2 = _ref2.length; _j < _len2; _j++) {
            constant = _ref2[_j];
            parameters.push(constant);
          }
          parameters.push(!this.outgoing);
          parameters.push(pattern);
          parameters.push(value);
          value = this.transforms[transform.name].apply(null, parameters);
        }
      }
      return value;
    };
    Packet.prototype.nextValue = function(value) {
      var bytes, little, pattern;
      pattern = this.pattern[this.patternIndex];
      little = pattern.endianness === 'l';
      bytes = pattern.bytes;
      this.value = value;
      this.offset = little ? 0 : bytes - 1;
      this.increment = little ? 1 : -1;
      return this.terminal = little ? bytes : -1;
    };
    Packet.prototype.unpack = function() {
      var bytes, i, pattern, top, value;
      bytes = this.value;
      if (bytes == null) {
        return null;
      }
      pattern = this.pattern[this.patternIndex];
      if (pattern.type === "h") {
        return hex(bytes.reverse());
      } else if (pattern.type === "f") {
        if (pattern.bits === 32) {
          return ieee754.fromIEEE754Single(bytes);
        } else {
          return ieee754.fromIEEE754Double(bytes);
        }
      } else if (pattern.signed) {
        value = 0;
        if ((bytes[bytes.length - 1] & 0x80) === 0x80) {
          top = bytes.length - 1;
          for (i = 0; (0 <= top ? i < top : i > top); (0 <= top ? i += 1 : i -= 1)) {
            value += (~bytes[i] & 0xff) * Math.pow(256, i);
          }
          value += (~(bytes[top] & 0x7f) & 0xff & 0x7f) * Math.pow(256, top);
          value += 1;
          value *= -1;
        } else {
          top = bytes.length - 1;
          for (i = 0; (0 <= top ? i < top : i > top); (0 <= top ? i += 1 : i -= 1)) {
            value += (bytes[i] & 0xff) * Math.pow(256, i);
          }
          value += (bytes[top] & 0x7f) * Math.pow(256, top);
        }
        return value;
      } else {
        return bytes;
      }
    };
    Packet.prototype.pack = function(value) {
      var pattern;
      pattern = this.pattern[this.patternIndex];
      if (pattern.type === "f") {
        if (pattern.bits === 32) {
          return ieee754.toIEEE754Single(value);
        } else {
          return ieee754.toIEEE754Double(value);
        }
      } else {
        return value;
      }
    };
    return Packet;
  })();
  bufferize = function(array) {
    var b, buffer, i, _len;
    buffer = new Buffer(array.length);
    for (i = 0, _len = array.length; i < _len; i++) {
      b = array[i];
      buffer[i] = b;
    }
    return buffer;
  };
  transforms = {
    str: function(encoding, parsing, field, value) {
      var buffer, i, length, _ref, _ref2;
      if (parsing) {
        if (!(value instanceof Buffer)) {
          value = bufferize(value);
        }
        if (/^ascii$/i.test(encoding)) {
          for (i = 0, _ref = value.length; (0 <= _ref ? i < _ref : i > _ref); (0 <= _ref ? i += 1 : i -= 1)) {
            if (value[i] & 0x80) {
              value[i] = 0;
            }
          }
          encoding = "utf8";
        }
        length = value.length;
        if (field.terminator) {
          length -= field.terminator.length;
        }
        return value.toString(encoding, 0, length);
      } else {
        if (field.terminator) {
          value += field.terminator;
        }
        buffer = new Buffer(value, encoding);
        if (encoding === "ascii") {
          for (i = 0, _ref2 = buffer.length; (0 <= _ref2 ? i < _ref2 : i > _ref2); (0 <= _ref2 ? i += 1 : i -= 1)) {
            if (value.charAt(i) === '\0') {
              buffer[i] = 0;
            }
          }
        }
        return buffer;
      }
    },
    ascii: function(parsing, field, value) {
      return transforms.str("ascii", parsing, field, value);
    },
    utf8: function(parsing, field, value) {
      return transforms.str("utf8", parsing, field, value);
    },
    pad: function(character, length, parsing, field, value) {
      if (!parsing) {
        while (value.length < length) {
          value = character + value;
        }
      }
      return value;
    },
    atoi: function(base, parsing, field, value) {
      if (parsing) {
        return parseInt(value, base);
      } else {
        return value.toString(base);
      }
    },
    atof: function(parsing, field, value) {
      if (parsing) {
        return parseFloat(value);
      } else {
        return value.toString();
      }
    }
  };
  module.exports.Parser = Parser = (function() {
    function Parser() {
      Parser.__super__.constructor.apply(this, arguments);
    }
    __extends(Parser, Packet);
    Parser.prototype.data = function() {
      var data;
      data = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return this.user = data;
    };
    Parser.prototype.getBytesRead = function() {
      return this.bytesRead;
    };
    Parser.prototype.nextField = function() {
      var pattern;
      pattern = this.pattern[this.patternIndex];
      this.repeat = pattern.repeat;
      this.index = 0;
      this.skipping = 0;
      this.terminated = !pattern.terminator;
      if (pattern.arrayed && pattern.endianness !== "x") {
        return this.fields.push([]);
      }
    };
    Parser.prototype.nextValue = function() {
      var bytes, little, pattern, reading, value;
      reading = !this.outgoing;
      pattern = this.pattern[this.patternIndex];
      if (pattern.endianness === "x") {
        return this.skipping = pattern.bytes;
      } else {
        little = pattern.endianness === "l";
        bytes = pattern.bytes;
        if (pattern.unpacked) {
          value = [];
        } else {
          value = 0;
        }
        return Parser.__super__.nextValue.call(this, value);
      }
    };
    Parser.prototype.parse = function(nameOrPattern, callback) {
      var packet, pattern;
      packet = this.packets[nameOrPattern] || {};
      pattern = packet.pattern || parsePattern(nameOrPattern);
      callback || (callback = packet.callback || noop);
      this.pattern = pattern;
      this.callback = callback;
      this.patternIndex = 0;
      this.fields = [];
      this.nextField();
      return this.nextValue();
    };
    Parser.prototype.read = function(buffer, offset, length) {
      var advance, b, end, field, p, start, _i, _len, _ref;
      offset || (offset = 0);
      length || (length = buffer.length);
      start = this.bytesRead;
      end = offset + length;
      b;
      while (this.pattern !== null && offset < end) {
        if (this.skipping) {
          advance = Math.min(this.skipping, end - offset);
          offset += advance;
          this.skipping -= advance;
          this.bytesRead += advance;
          if (this.skipping) {
            return this.bytesRead - start;
          }
        } else {
          if (this.pattern[this.patternIndex].unpacked) {
            while (true) {
              b = buffer[offset];
              this.bytesRead++;
              offset++;
              this.value[this.offset] = b;
              this.offset += this.increment;
              if (this.offset === this.terminal) {
                break;
              }
              if (offset === end) {
                return this.bytesRead - start;
              }
            }
          } else if (this.pattern[this.patternIndex].arrayed && this.pattern[this.patternIndex].repeat === 0) {
            this.value = null;
            this.terminated = true;
          } else {
            while (true) {
              b = buffer[offset];
              this.bytesRead++;
              offset++;
              this.value += Math.pow(256, this.offset) * b;
              this.offset += this.increment;
              if (this.offset === this.terminal) {
                break;
              }
              if (offset === end) {
                return this.bytesRead - start;
              }
            }
          }
          field = this.unpack();
          if (this.pattern[this.patternIndex].arrayed) {
            if (field != null) {
              this.fields[this.fields.length - 1].push(field);
            }
          } else {
            this.fields.push(field);
          }
        }
        if (!this.terminated) {
          if (this.pattern[this.patternIndex].terminator.charCodeAt(0) === field) {
            this.terminated = true;
            if (this.repeat === Number.MAX_VALUE) {
              this.repeat = this.index + 1;
            } else {
              this.skipping = (this.repeat - (++this.index)) * this.pattern[this.patternIndex].bytes;
              if (this.skipping) {
                this.repeat = this.index + 1;
                continue;
              }
            }
          }
        }
        if (++this.index < this.repeat) {
          this.nextValue();
        } else if (++this.patternIndex === this.pattern.length) {
          this.fields.push(this.pipeline(this.pattern[this.patternIndex - 1], this.fields.pop()));
          this.fields.push(this);
          _ref = this.user || [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            p = _ref[_i];
            this.fields.push(p);
          }
          this.pattern = null;
          this.callback.apply(null, this.fields);
        } else {
          if (this.pattern[this.patternIndex - 1].endianness !== "x") {
            if (this.pattern[this.patternIndex - 1].length) {
              this.pattern[this.patternIndex].repeat = this.fields.pop();
            } else {
              this.fields.push(this.pipeline(this.pattern[this.patternIndex - 1], this.fields.pop()));
            }
          }
          this.nextField();
          this.nextValue();
        }
      }
      return this.bytesRead - start;
    };
    return Parser;
  })();
  module.exports.Serializer = Serializer = (function() {
    function Serializer() {
      Serializer.__super__.constructor.apply(this, arguments);
    }
    __extends(Serializer, Packet);
    Serializer.prototype.getBytesWritten = function() {
      return this.bytesWritten;
    };
    Serializer.prototype.packet = function(name, pattern, callback) {
      var part, _i, _len, _ref, _results;
      Serializer.__super__.packet.call(this, name, pattern, callback);
      _ref = this.packets[name].pattern;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        part = _ref[_i];
        _results.push(part.transforms ? part.transforms.reverse() : void 0);
      }
      return _results;
    };
    Serializer.prototype.nextField = function() {
      var pattern;
      pattern = this.pattern[this.patternIndex];
      this.repeat = pattern.repeat;
      this.terminated = !pattern.terminator;
      this.index = 0;
      delete this.padding;
      if (pattern.endianness === "x") {
        this.outgoing.splice(this.patternIndex, 0, null);
        if (pattern.padding != null) {
          return this.padding = pattern.padding;
        }
      }
    };
    Serializer.prototype.nextValue = function() {
      var pattern, repeat, value;
      pattern = this.pattern[this.patternIndex];
      if (pattern.endianness === "x" && !(this.padding != null)) {
        return this.skipping = pattern.bytes;
      } else {
        if (this.padding != null) {
          value = this.padding;
        } else if (pattern.arrayed) {
          value = this.outgoing[this.patternIndex][this.index];
        } else {
          if (pattern.length) {
            repeat = this.outgoing[this.patternIndex].length;
            this.outgoing.splice(this.patternIndex, 0, repeat);
            this.pattern[this.patternIndex + 1].repeat = repeat;
          }
          value = this.outgoing[this.patternIndex];
        }
        if (pattern.unpacked) {
          value = this.pack(value);
        }
        return Serializer.__super__.nextValue.call(this, value);
      }
    };
    Serializer.prototype.serialize = function() {
      var callback, i, nameOrPattern, packet, part, pattern, shiftable, value, _i, _len, _len2, _ref;
      shiftable = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      if (typeof shiftable[shiftable.length - 1] === 'function') {
        callback = shiftable.pop();
      }
      nameOrPattern = shiftable.shift();
      packet = this.packets[nameOrPattern] || {};
      pattern = packet.pattern || parsePattern(nameOrPattern);
      callback || (callback = packet.callback || noop);
      if (!packet.pattern) {
        for (_i = 0, _len = pattern.length; _i < _len; _i++) {
          part = pattern[_i];
          if (part.transforms) {
            part.transforms.reverse();
          }
        }
      }
      this.pattern = pattern;
      this.callback = callback;
      this.patternIndex = 0;
      this.outgoing = shiftable;
      _ref = this.outgoing;
      for (i = 0, _len2 = _ref.length; i < _len2; i++) {
        value = _ref[i];
        this.outgoing[i] = this.pipeline(pattern[i], value);
      }
      this.nextField();
      return this.nextValue();
    };
    Serializer.prototype.write = function(buffer, offset, length) {
      var advance, end, start;
      offset || (offset = 0);
      length || (length = buffer.length);
      start = offset;
      end = offset + length;
      while (this.pattern && offset < end) {
        if (this.skipping) {
          advance = Math.min(this.skipping, end - offset);
          offset += advance;
          this.skipping -= advance;
          this.bytesWritten += advance;
          if (this.skipping) {
            return offset - start;
          }
        } else {
          if (this.pattern[this.patternIndex].unpacked) {
            while (true) {
              buffer[offset] = this.value[this.offset];
              this.offset += this.increment;
              this.bytesWritten++;
              offset++;
              if (this.offset === this.terminal) {
                break;
              }
              if (offset === end) {
                return offset - start;
              }
            }
          } else {
            while (true) {
              buffer[offset] = Math.floor(this.value / Math.pow(256, this.offset)) & 0xff;
              this.offset += this.increment;
              this.bytesWritten++;
              offset++;
              if (this.offset === this.terminal) {
                break;
              }
              if (offset === end) {
                return offset - start;
              }
            }
          }
        }
        if (!this.terminated) {
          if (this.pattern[this.patternIndex].terminator.charCodeAt(0) === this.value) {
            this.terminated = true;
            if (this.repeat === Number.MAX_VALUE) {
              this.repeat = this.index + 1;
            } else if (this.pattern[this.patternIndex].padding != null) {
              this.padding = this.pattern[this.patternIndex].padding;
            } else {
              this.skipping = (this.repeat - (++this.index)) * this.pattern[this.patternIndex].bytes;
              if (this.skipping) {
                this.repeat = this.index + 1;
                continue;
              }
            }
          }
        }
        if (++this.index < this.repeat) {
          this.nextValue();
        } else if (++this.patternIndex === this.pattern.length) {
          this.pattern = null;
          this.callback.apply(null, [this]);
        } else {
          delete this.padding;
          this.repeat = this.pattern[this.patternIndex].repeat;
          this.terminated = !this.pattern[this.patternIndex].terminator;
          this.index = 0;
          this.nextValue();
        }
      }
      this.outgoing = null;
      return offset - start;
    };
    return Serializer;
  })();
  module.exports.Structure = Structure = (function() {
    function Structure(pattern) {
      this.parser = new Parser();
      this.parser.packet("structure", pattern);
      this.serializer = new Serializer();
      this.serializer.packet("structure", pattern);
    }
    Structure.prototype.sizeOf = function() {
      var values;
      values = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return 0;
    };
    Structure.prototype.read = function(buffer, offset, callback) {
      if (typeof offset === "function" && !callback) {
        callback = offset;
      }
      this.parser.reset();
      this.parser.parse("structure", callback);
      return this.parser.read(buffer, offset, Number.MAX_VALUE);
    };
    Structure.prototype.write = function() {
      var buffer, values;
      values = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return buffer = values.shift();
    };
    return Structure;
  })();
}).call(this);
