(function() {
  var packing, scalar;
  scalar = /('(?:[^\\']|\\.)+'|"(?:[^\\"]|\\.)+"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?)(\s*,\s*|\s*\))?(.*)/;
  packing = function(pattern) {
    var fields, rest;
    fields = [];
    return rest = pattern;
  };
  module.exports.parse = function(pattern) {
    var arg, arrayed, base, f, fields, hasArgument, index, length, lengthEncoded, match, more, pack, pad, padding, pipe, rest, transform, tz, value;
    fields = [];
    lengthEncoded = false;
    rest = pattern;
    index = 0;
    while (true) {
      match = /^(-?)([xbl])(\d+)([fha]?)(.*)$/.exec(rest);
      if (!match) {
        throw new Error("invalid pattern at " + index);
      }
      f = {
        signed: !!match[1] || match[4] === "f",
        endianness: match[2] === 'n' ? 'b' : match[2],
        bits: parseInt(match[3], 10),
        type: match[4] || 'n'
      };
      rest = match[5];
      if (f.signed) {
        index++;
      }
      index++;
      if (f.bits === 0 || f.bits % 8) {
        throw new Error("bits must be divisible by 8 at " + index);
      }
      if (f.type === "f" && !(f.bits === 32 || f.bits === 64)) {
        throw Error("floats can only be 32 or 64 bits at " + index);
      }
      index += match[3].length;
      if (match[4]) {
        index++;
      }
      if (f.bits > 64 && f.type === "n") {
        f.type = "a";
      }
      f.bytes = f.bits / 8;
      f.unpacked = f.signed || f.bytes > 8 || "ha".indexOf(f.type) !== -1;
      pack = /^{([^b-][^}]+)}(.*)$/.exec(rest);
      if (pack) {
        f.unpacked = true;
        f.packing = packing(pack[1]);
        rest = pack[2];
      }
      length = /^\/(.*)$/.exec(rest);
      if (length) {
        f.length = true;
        rest = length[1];
        f.arrayed = false;
        f.repeat = 1;
        lengthEncoded = true;
        fields.push(f);
        continue;
      }
      f.repeat = 1;
      f.arrayed = lengthEncoded;
      if (!lengthEncoded) {
        arrayed = /^\[(\d+)\](.*)$/.exec(rest);
        if (arrayed) {
          f.arrayed = true;
          f.repeat = parseInt(arrayed[1], 10);
          index++;
          if (f.repeat === 0) {
            throw new Error("array length must be non-zero at " + index);
          }
          index += arrayed[1].length + 1;
          rest = arrayed[2];
        }
      }
      padding = /^{(0x|0)?(\d+)}(.*)$/.exec(rest);
      if (padding) {
        base = padding[1];
        pad = padding[2];
        rest = padding[3];
        if (base) {
          if (base === "0x") {
            f.padding = parseInt(pad, 16);
          } else {
            f.padding = parseInt(pad, 8);
          }
        } else {
          f.padding = parseInt(pad, 10);
        }
      }
      tz = /^z(?:<(.*?)>)?(.*)$/.exec(rest);
      if (tz) {
        f.terminator = tz[1] || "\0";
        f.arrayed = true;
        rest = tz[2];
        if (f.repeat === 1) {
          f.repeat = Number.MAX_VALUE;
        }
      }
      while (pipe = /^\|(\w[\w\d]*)\((\)?)(.*)/.exec(rest)) {
        transform = {
          name: pipe[1],
          parameters: []
        };
        rest = pipe[3];
        hasArgument = !pipe[2];
        while (hasArgument) {
          arg = scalar.exec(rest);
          value = eval(arg[1]);
          hasArgument = arg[2].indexOf(")") === -1;
          rest = arg[3];
          transform.parameters.push(value);
        }
        (f.transforms || (f.transforms = [])).push(transform);
      }
      fields.push(f);
      more = /\s*,\s*(.*)/.exec(rest);
      if (!more) {
        break;
      }
      rest = more[1];
      lengthEncoded = false;
    }
    if (/\S/.test(rest)) {
      throw new Error("invalid pattern at " + index);
    }
    return fields;
  };
}).call(this);
